import { EnvContext } from '@typed-macro/core';
import { InstantiatedMacroProvider } from '@typed-macro/core';
import { Macro } from '@typed-macro/core';
import { MacroProvider } from '@typed-macro/core';
import { MacroProviderHooks } from '@typed-macro/core';
import { ModularizedExportable } from '@typed-macro/core';
import { ParserPlugin } from '@babel/parser';

export declare type CombinedHooks = {
    onStart(): Promise<void>;
    onFilter(id: string): Promise<boolean>;
    beforeTransform(code: string, id: string): Promise<string>;
    afterTransform(code: string, id: string): Promise<string>;
    onStop(): Promise<void>;
};

export declare function createFilter({ include, exclude, }: FilterOptions): Filter;

export declare function createHookContainer(): HookContainer;

export declare function createLoader(): Loader;

export declare function createRuntime(env: EnvContext, options: RuntimeOptions): Runtime;

export declare function createTransformer(options: TransformerOptions): Transformer_2;

export declare function createTypeRenderer(): TypeRenderer;

export declare type Filter = {
    /**
     * Check if is included.
     */
    isIncluded(id: string): boolean;
    /**
     * Check if is excluded.
     */
    isExcluded(id: string): boolean;
};

export declare type FilterOptions = {
    /**
     * Regexp or glob patterns to include matches, applied after
     * {@link FilterOptions.exclude}.
     *
     * Once this option is defined, the default one will be overwritten.
     *
     * @default /\.[jt]sx?$/
     */
    include?: FilterPattern;
    /**
     * Regexp or glob patterns to exclude matches, applied before
     * {@link FilterOptions.include}.
     *
     * Once this option is defined, the default one will be overwritten.
     *
     * @default /node_modules/
     */
    exclude?: FilterPattern;
};

export declare type FilterPattern = RegExp | string | Array<RegExp | string>;

export declare type HookContainer = CombinedHooks & {
    /**
     * Append hooks into container.
     */
    append(hooks: MacroProviderHooks): void;
};

export declare type Loader = {
    /**
     * Append code into specific module.
     *
     * When code is undefined, use empty module `export {}` by default.
     *
     * Note that one module can only have one code,
     * and appending code for one module twice will raise an exception.
     */
    append(moduleName: string, code?: string): void;
    /**
     * Get the code in specific module.
     */
    load(moduleName: string): string | undefined;
};

/* Excluded from this release type: ModularizedMacros */

/* Excluded from this release type: ModularizedModules */

/* Excluded from this release type: ModularizedTypes */

/* Excluded from this release type: NormalizedExports */

/* Excluded from this release type: NormalizedMacroProvider */

/* Excluded from this release type: normalizeExports */

/* Excluded from this release type: normalizeMacroProvider */

export declare type Runtime = {
    /**
     * Append macro providers into runtime.
     */
    appendProvider(provider: MacroProvider): void;
    /**
     * Render types into file.
     */
    renderTypes(filepath: string): Promise<void>;
    /**
     * Call onStart hooks.
     */
    start(): Promise<void>;
    /**
     * Call onStop hooks.
     */
    stop(): Promise<void>;
    /**
     * Process file transformation, including calling transform-related hooks
     * and macro transformation.
     */
    transform(code: string, id: string): Promise<string>;
    /**
     * Check is file should be transformed.
     */
    filter(id: string): Promise<boolean>;
    /**
     * Load code by module name.
     */
    load(id: string): string | undefined;
    /**
     * Check if the virtual module exists.
     */
    resolveId(id: string): string | undefined;
    /* Excluded from this release type: internal */
};

export declare type RuntimeOptions = {
    /**
     * @see FilterOptions
     */
    filter?: FilterOptions;
    /**
     * @see TransformerOptions
     */
    transformer?: TransformerOptions;
};

declare type Transformer_2 = {
    /**
     * Append macros into transformer.
     */
    appendMacros(moduleName: string, macros: Macro[]): void;
    /**
     * Append parser plugins into transformer.
     * @see MacroProviderOptions.parserPlugins
     */
    appendParserPlugins(plugins: ParserPlugin[]): void;
    /**
     * Transform a file.
     *
     * Returns undefined if no macro applied.
     */
    transform(code: string, id: string, env: EnvContext): string | undefined;
};
export { Transformer_2 as Transformer }

export declare type TransformerOptions = {
    /**
     * The max traversal times for applying macros.
     *
     * An error will be thrown if reached the maxTraversals. It's usually caused
     * by forgetting to remove/modify the call expression in macros.
     *
     * @default 5
     */
    maxTraversals?: number;
    /**
     * @see MacroProviderOptions.parserPlugins
     */
    parserPlugins?: ParserPlugin[];
};

export declare type TypeRenderer = {
    /**
     * Render types into string.
     */
    renderToString(): string;
    /**
     * Render types into file.
     */
    renderToFile(path: string): Promise<void>;
    /**
     * Append types into specific module.
     */
    append(moduleName: string, types: string[]): void;
};

export { }
